import{n as o,q as w,T as p,v as N,w as h,x as f,y as i,E as y,z as C,A as m,B as x,C as A}from"./NqQ1dWOy.js";function E(r){var n=document.createElement("template");return n.innerHTML=r.replaceAll("<!>","<!---->"),n.content}function t(r,n){var e=h;e.nodes===null&&(e.nodes={start:r,end:n,a:null,t:null})}function L(r,n){var e=(n&p)!==0,l=(n&N)!==0,s,v=!r.startsWith("<!>");return()=>{if(f)return t(i,null),i;s===void 0&&(s=E(v?r:"<!>"+r),e||(s=o(s)));var a=l||w?document.importNode(s,!0):s.cloneNode(!0);if(e){var _=o(a),d=a.lastChild;t(_,d)}else t(a,a);return a}}function F(r,n,e="svg"){var l=!r.startsWith("<!>"),s=(n&p)!==0,v=`<${e}>${l?r:"<!>"+r}</${e}>`,a;return()=>{if(f)return t(i,null),i;if(!a){var _=E(v),d=o(_);if(s)for(a=document.createDocumentFragment();o(d);)a.appendChild(o(d));else a=o(d)}var u=a.cloneNode(!0);if(s){var T=o(u),g=u.lastChild;t(T,g)}else t(u,u);return u}}function O(r,n){return F(r,n,"svg")}function P(r=""){if(!f){var n=m(r+"");return t(n,n),n}var e=i;return e.nodeType!==x&&(e.before(e=m()),A(e)),t(e,e),e}function R(){if(f)return t(i,null),i;var r=document.createDocumentFragment(),n=document.createComment(""),e=m();return r.append(n,e),t(n,e),r}function b(r,n){if(f){var e=h;((e.f&y)===0||e.nodes.end===null)&&(e.nodes.end=i),C();return}r!==null&&r.before(n)}const M="5";var c;typeof window<"u"&&((c=window.__svelte??(window.__svelte={})).v??(c.v=new Set)).add(M);export{b as a,t as b,R as c,E as d,O as e,L as f,P as t};
