import{f as h,a as g}from"../chunks/C0Fr8dve.js";import{i as l}from"../chunks/BSNlmTZj.js";import{p as v,m as d,t as _,i as x,j as e,k as o,s as k}from"../chunks/NqQ1dWOy.js";import{s as p}from"../chunks/R8CEIRAd.js";import{s as $,p as m}from"../chunks/B8AwtY2H.js";const b={get error(){return m.error},get status(){return m.status}};$.updated.check;const i=b;var j=h("<h1> </h1> <p> </p>",1);function A(f,n){v(n,!1),l();var t=j(),r=d(t),c=e(r,!0);o(r);var s=k(r,2),u=e(s,!0);o(s),_(()=>{var a;p(c,i.status),p(u,(a=i.error)==null?void 0:a.message)}),g(f,t),x()}export{A as component};
